/*
 * main.h
 *
 * PWLib application header file for playwav
 *
 * A H.323 answering application
 *
 * Copyright (c) 2005 Nickolay V. Shmyrev <nshmyrev@yandex.ru>
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: main.h,v $
 *
 * Revision 1.2  2005/10/01 05:40:18  nshmyrev
 * More clean code
 *
 * Revision 1.1  2005/09/11 00:15:18  nshmyrev
 * Initial version
 *
 *
 */

#ifndef _PLAYWAV_MAIN_H
#define _PLAYWAV_MAIN_H

#include <h323.h>
#include <h323pdu.h>
#include <h245.h>
#include <lid.h>

#include <ptclib/pwavfile.h>
#include <ptclib/delaychan.h>

class PlayWav : public PProcess
{
  PCLASSINFO(PlayWav, PProcess)

  public:
    PlayWav();
    ~PlayWav();

    void Main();

  protected:
    long GetCodec(const PString & codecname);
    OpalLineInterfaceDevice * GetDevice(const PString & device);
};

class MyH323EndPoint : public H323EndPoint
{
  PCLASSINFO(MyH323EndPoint, H323EndPoint);

  public:
    // overrides from H323EndPoint
    virtual H323Connection * CreateConnection(unsigned callReference);
    BOOL OnIncomingCall(H323Connection &, const H323SignalPDU &, H323SignalPDU &);

    // new functions
    BOOL Initialise(PConfigArgs & args);

    enum {
      NoHangupAfterPlay   = 0x01,
      LoopMessage       = 0x02
    };

    PString    GetOgmFile() const            { return ogmFile; }
    void       SetOgmFile(const PString & s) { ogmFile = s; }

    PString    GetRunCmd() const            { return runCmd; }
    void       SetRunCmd(const PString & s) { runCmd = s; }

    BOOL GetHangupAfterPlay() const     { return !(flags & NoHangupAfterPlay); }
    BOOL GetLoopMessage    () const     { return flags & LoopMessage; }

  protected:
    PString runCmd;
    PString ogmFile;
    int flags;
};

class MyH323Connection;
PQUEUE(PStringQueue, PString);

class PCM_OGMChannel : public PIndirectChannel
{
  PCLASSINFO(PCM_OGMChannel, PIndirectChannel);

  public:
    PCM_OGMChannel(MyH323Connection & conn);
    BOOL Read(void * buffer, PINDEX amount);
    BOOL Close();

    void QueueFile(const PString & cmd);

    void SetPlayOnce() { playOnce = TRUE; }
    void SetLoopMessage() { loopMessage = TRUE; }

  protected:
    virtual BOOL ReadFrame(PINDEX amount);
    virtual void CreateSilenceFrame(PINDEX amount);
    virtual void Synchronise(PINDEX amount);
    virtual BOOL IsWAVFileValid(PWAVFile *chan);

    BOOL AdjustFrame(void * buffer, PINDEX amount);

    PStringQueue playQueue;

    MyH323Connection & conn;
    PMutex chanMutex;
    int silentCount;
    int totalData;
    BOOL closed;
    BOOL playOnce;
    BOOL loopMessage;
    BOOL finished;

    PAdaptiveDelay ogm_delay;
    PBYTEArray frameBuffer;
    PINDEX frameLen, frameOffs;
};

class MyH323Connection : public H323Connection
{
  PCLASSINFO(MyH323Connection, H323Connection);

  public:
    MyH323Connection(MyH323EndPoint &, unsigned, unsigned);
    ~MyH323Connection();

    // overrides from H323Connection
    BOOL OnCreateLogicalChannel(const H323Capability & capability,
                                H323Channel::Directions dir,
                                unsigned & errorCode);
    BOOL OpenAudioChannel(BOOL, unsigned, H323AudioCodec & codec);
    AnswerCallResponse OnAnswerCall(const PString &, const H323SignalPDU &, H323SignalPDU &);
    
    // new functions
    void PlayCommand ();
    void Hangup();

  protected:
    const MyH323EndPoint & ep;
    PMutex  connMutex;

    // Log and script params.
    PTime   callStartTime;
    PString transmitCodecName;

    PString calledParty;
    PString callerParty;
    PString callerID;
    
    PCM_OGMChannel * ogmChannel;
};

#endif  // PLAYWAV_H_


// End of File ///////////////////////////////////////////////////////////////
